// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kNormal = 56,
  kBold = 57,
  kBolder = 58,
  kLighter = 59,
  kUltraCondensed = 60,
  kExtraCondensed = 61,
  kCondensed = 62,
  kSemiCondensed = 63,
  kSemiExpanded = 64,
  kExpanded = 65,
  kExtraExpanded = 66,
  kUltraExpanded = 67,
  kXxSmall = 68,
  kXSmall = 69,
  kSmall = 70,
  kMedium = 71,
  kLarge = 72,
  kXLarge = 73,
  kXxLarge = 74,
  kXxxLarge = 75,
  kSmaller = 76,
  kLarger = 77,
  kWebkitXxxLarge = 78,
  kSerif = 79,
  kSansSerif = 80,
  kCursive = 81,
  kFantasy = 82,
  kMonospace = 83,
  kWebkitBody = 84,
  kWebkitPictograph = 85,
  kSwap = 86,
  kFallback = 87,
  kOptional = 88,
  kAqua = 89,
  kBlack = 90,
  kBlue = 91,
  kFuchsia = 92,
  kGray = 93,
  kGreen = 94,
  kLime = 95,
  kMaroon = 96,
  kNavy = 97,
  kOlive = 98,
  kOrange = 99,
  kPurple = 100,
  kRed = 101,
  kSilver = 102,
  kTeal = 103,
  kWhite = 104,
  kYellow = 105,
  kTransparent = 106,
  kWebkitLink = 107,
  kWebkitActivelink = 108,
  kActiveborder = 109,
  kActivecaption = 110,
  kAppworkspace = 111,
  kBackground = 112,
  kButtonface = 113,
  kButtonhighlight = 114,
  kButtonshadow = 115,
  kButtontext = 116,
  kCaptiontext = 117,
  kGraytext = 118,
  kHighlight = 119,
  kHighlighttext = 120,
  kInactiveborder = 121,
  kInactivecaption = 122,
  kInactivecaptiontext = 123,
  kInfobackground = 124,
  kInfotext = 125,
  kLinktext = 126,
  kMenutext = 127,
  kScrollbar = 128,
  kThreeddarkshadow = 129,
  kThreedface = 130,
  kThreedhighlight = 131,
  kThreedlightshadow = 132,
  kThreedshadow = 133,
  kVisitedtext = 134,
  kWindow = 135,
  kWindowframe = 136,
  kWindowtext = 137,
  kInternalActiveListBoxSelection = 138,
  kInternalActiveListBoxSelectionText = 139,
  kInternalInactiveListBoxSelection = 140,
  kInternalInactiveListBoxSelectionText = 141,
  kWebkitFocusRingColor = 142,
  kCurrentcolor = 143,
  kGrey = 144,
  kInternalQuirkInherit = 145,
  kInternalRootColor = 146,
  kRepeat = 147,
  kRepeatX = 148,
  kRepeatY = 149,
  kNoRepeat = 150,
  kClear = 151,
  kCopy = 152,
  kSourceOver = 153,
  kSourceIn = 154,
  kSourceOut = 155,
  kSourceAtop = 156,
  kDestinationOver = 157,
  kDestinationIn = 158,
  kDestinationOut = 159,
  kDestinationAtop = 160,
  kXor = 161,
  kPlusLighter = 162,
  kBaseline = 163,
  kMiddle = 164,
  kSub = 165,
  kSuper = 166,
  kTextTop = 167,
  kTextBottom = 168,
  kTop = 169,
  kBottom = 170,
  kWebkitBaselineMiddle = 171,
  kWebkitAuto = 172,
  kLeft = 173,
  kRight = 174,
  kInlineStart = 175,
  kInlineEnd = 176,
  kCenter = 177,
  kJustify = 178,
  kWebkitLeft = 179,
  kWebkitRight = 180,
  kWebkitCenter = 181,
  kWebkitMatchParent = 182,
  kInternalCenter = 183,
  kInterWord = 184,
  kDistribute = 185,
  kOutside = 186,
  kInside = 187,
  kDisc = 188,
  kCircle = 189,
  kSquare = 190,
  kDecimal = 191,
  kDecimalLeadingZero = 192,
  kArabicIndic = 193,
  kBengali = 194,
  kCambodian = 195,
  kKhmer = 196,
  kDevanagari = 197,
  kGujarati = 198,
  kGurmukhi = 199,
  kKannada = 200,
  kLao = 201,
  kMalayalam = 202,
  kMongolian = 203,
  kMyanmar = 204,
  kOriya = 205,
  kPersian = 206,
  kUrdu = 207,
  kTelugu = 208,
  kTibetan = 209,
  kThai = 210,
  kLowerRoman = 211,
  kUpperRoman = 212,
  kLowerGreek = 213,
  kLowerAlpha = 214,
  kLowerLatin = 215,
  kUpperAlpha = 216,
  kUpperLatin = 217,
  kCjkEarthlyBranch = 218,
  kCjkHeavenlyStem = 219,
  kEthiopicHalehame = 220,
  kEthiopicHalehameAm = 221,
  kEthiopicHalehameTiEr = 222,
  kEthiopicHalehameTiEt = 223,
  kHangul = 224,
  kHangulConsonant = 225,
  kKoreanHangulFormal = 226,
  kKoreanHanjaFormal = 227,
  kKoreanHanjaInformal = 228,
  kHebrew = 229,
  kArmenian = 230,
  kLowerArmenian = 231,
  kUpperArmenian = 232,
  kGeorgian = 233,
  kCjkIdeographic = 234,
  kSimpChineseFormal = 235,
  kSimpChineseInformal = 236,
  kTradChineseFormal = 237,
  kTradChineseInformal = 238,
  kHiragana = 239,
  kKatakana = 240,
  kHiraganaIroha = 241,
  kKatakanaIroha = 242,
  kInline = 243,
  kBlock = 244,
  kFlowRoot = 245,
  kListItem = 246,
  kInlineBlock = 247,
  kTable = 248,
  kInlineTable = 249,
  kTableRowGroup = 250,
  kTableHeaderGroup = 251,
  kTableFooterGroup = 252,
  kTableRow = 253,
  kTableColumnGroup = 254,
  kTableColumn = 255,
  kTableCell = 256,
  kTableCaption = 257,
  kWebkitBox = 258,
  kWebkitInlineBox = 259,
  kFlex = 260,
  kInlineFlex = 261,
  kGrid = 262,
  kInlineGrid = 263,
  kContents = 264,
  kWebkitFlex = 265,
  kWebkitInlineFlex = 266,
  kLayout = 267,
  kInlineLayout = 268,
  kAuto = 269,
  kCrosshair = 270,
  kDefault = 271,
  kPointer = 272,
  kMove = 273,
  kVerticalText = 274,
  kCell = 275,
  kContextMenu = 276,
  kAlias = 277,
  kProgress = 278,
  kNoDrop = 279,
  kNotAllowed = 280,
  kZoomIn = 281,
  kZoomOut = 282,
  kEResize = 283,
  kNeResize = 284,
  kNwResize = 285,
  kNResize = 286,
  kSeResize = 287,
  kSwResize = 288,
  kSResize = 289,
  kWResize = 290,
  kEwResize = 291,
  kNsResize = 292,
  kNeswResize = 293,
  kNwseResize = 294,
  kColResize = 295,
  kRowResize = 296,
  kText = 297,
  kWait = 298,
  kHelp = 299,
  kAllScroll = 300,
  kGrab = 301,
  kGrabbing = 302,
  kWebkitGrab = 303,
  kWebkitGrabbing = 304,
  kWebkitZoomIn = 305,
  kWebkitZoomOut = 306,
  kLtr = 307,
  kRtl = 308,
  kCapitalize = 309,
  kUppercase = 310,
  kLowercase = 311,
  kVisible = 312,
  kCollapse = 313,
  kA3 = 314,
  kA4 = 315,
  kA5 = 316,
  kAbove = 317,
  kAbsolute = 318,
  kAlways = 319,
  kAvoid = 320,
  kB4 = 321,
  kB5 = 322,
  kBelow = 323,
  kBidiOverride = 324,
  kBlink = 325,
  kBoth = 326,
  kBreakSpaces = 327,
  kCloseQuote = 328,
  kEmbed = 329,
  kFixed = 330,
  kHand = 331,
  kHide = 332,
  kIsolate = 333,
  kIsolateOverride = 334,
  kPlaintext = 335,
  kWebkitIsolate = 336,
  kWebkitIsolateOverride = 337,
  kWebkitPlaintext = 338,
  kLandscape = 339,
  kLedger = 340,
  kLegal = 341,
  kLetter = 342,
  kLineThrough = 343,
  kLocal = 344,
  kNoCloseQuote = 345,
  kNoOpenQuote = 346,
  kNowrap = 347,
  kOpenQuote = 348,
  kOverlay = 349,
  kOverline = 350,
  kPortrait = 351,
  kPre = 352,
  kPreLine = 353,
  kPreWrap = 354,
  kRelative = 355,
  kScroll = 356,
  kSeparate = 357,
  kShow = 358,
  kStatic = 359,
  kThick = 360,
  kThin = 361,
  kUnderline = 362,
  kWavy = 363,
  kWebkitNowrap = 364,
  kStretch = 365,
  kStart = 366,
  kEnd = 367,
  kClone = 368,
  kSlice = 369,
  kReverse = 370,
  kHorizontal = 371,
  kVertical = 372,
  kInlineAxis = 373,
  kBlockAxis = 374,
  kFlexStart = 375,
  kFlexEnd = 376,
  kSpaceBetween = 377,
  kSpaceAround = 378,
  kSpaceEvenly = 379,
  kUnsafe = 380,
  kSafe = 381,
  kRow = 382,
  kRowReverse = 383,
  kColumn = 384,
  kColumnReverse = 385,
  kWrap = 386,
  kWrapReverse = 387,
  kAutoFlow = 388,
  kDense = 389,
  kReadOnly = 390,
  kReadWrite = 391,
  kReadWritePlaintextOnly = 392,
  kElement = 393,
  kWebkitMinContent = 394,
  kWebkitMaxContent = 395,
  kWebkitFillAvailable = 396,
  kWebkitFitContent = 397,
  kMinContent = 398,
  kMaxContent = 399,
  kFitContent = 400,
  kClip = 401,
  kEllipsis = 402,
  kDiscard = 403,
  kBreakAll = 404,
  kKeepAll = 405,
  kBreakWord = 406,
  kSpace = 407,
  kLoose = 408,
  kStrict = 409,
  kAfterWhiteSpace = 410,
  kAnywhere = 411,
  kManual = 412,
  kCheckbox = 413,
  kRadio = 414,
  kPushButton = 415,
  kSquareButton = 416,
  kButton = 417,
  kInnerSpinButton = 418,
  kListbox = 419,
  kMediaSlider = 420,
  kMediaSliderthumb = 421,
  kMediaVolumeSlider = 422,
  kMediaVolumeSliderthumb = 423,
  kInternalMediaControl = 424,
  kMenulist = 425,
  kMenulistButton = 426,
  kMeter = 427,
  kProgressBar = 428,
  kSliderHorizontal = 429,
  kSliderVertical = 430,
  kSliderthumbHorizontal = 431,
  kSliderthumbVertical = 432,
  kSearchfield = 433,
  kSearchfieldCancelButton = 434,
  kTextfield = 435,
  kTextarea = 436,
  kRound = 437,
  kBorder = 438,
  kBorderBox = 439,
  kContent = 440,
  kContentBox = 441,
  kPadding = 442,
  kPaddingBox = 443,
  kMarginBox = 444,
  kContain = 445,
  kCover = 446,
  kLogical = 447,
  kVisual = 448,
  kAlternate = 449,
  kAlternateReverse = 450,
  kForwards = 451,
  kBackwards = 452,
  kInfinite = 453,
  kRunning = 454,
  kPaused = 455,
  kFlat = 456,
  kPreserve3d = 457,
  kFillBox = 458,
  kViewBox = 459,
  kEase = 460,
  kLinear = 461,
  kEaseIn = 462,
  kEaseOut = 463,
  kEaseInOut = 464,
  kJumpBoth = 465,
  kJumpEnd = 466,
  kJumpNone = 467,
  kJumpStart = 468,
  kStepStart = 469,
  kStepEnd = 470,
  kSteps = 471,
  kFrames = 472,
  kCubicBezier = 473,
  kDocument = 474,
  kReset = 475,
  kZoom = 476,
  kVisiblepainted = 477,
  kVisiblefill = 478,
  kVisiblestroke = 479,
  kPainted = 480,
  kFill = 481,
  kStroke = 482,
  kBoundingBox = 483,
  kSpellOut = 484,
  kDigits = 485,
  kLiteralPunctuation = 486,
  kNoPunctuation = 487,
  kAntialiased = 488,
  kSubpixelAntialiased = 489,
  kOptimizespeed = 490,
  kOptimizelegibility = 491,
  kGeometricprecision = 492,
  kCrispedges = 493,
  kEconomy = 494,
  kExact = 495,
  kLr = 496,
  kRl = 497,
  kTb = 498,
  kLrTb = 499,
  kRlTb = 500,
  kTbRl = 501,
  kHorizontalTb = 502,
  kVerticalRl = 503,
  kVerticalLr = 504,
  kAfter = 505,
  kBefore = 506,
  kOver = 507,
  kUnder = 508,
  kFilled = 509,
  kOpen = 510,
  kDot = 511,
  kDoubleCircle = 512,
  kTriangle = 513,
  kSesame = 514,
  kEllipse = 515,
  kClosestSide = 516,
  kClosestCorner = 517,
  kFarthestSide = 518,
  kFarthestCorner = 519,
  kMixed = 520,
  kSideways = 521,
  kSidewaysRight = 522,
  kUpright = 523,
  kVerticalRight = 524,
  kOn = 525,
  kOff = 526,
  kOptimizequality = 527,
  kPixelated = 528,
  kWebkitOptimizeContrast = 529,
  kFromImage = 530,
  kNonzero = 531,
  kEvenodd = 532,
  kAt = 533,
  kAlphabetic = 534,
  kFullscreen = 535,
  kStandalone = 536,
  kMinimalUi = 537,
  kBrowser = 538,
  kSticky = 539,
  kCoarse = 540,
  kFine = 541,
  kOnDemand = 542,
  kHover = 543,
  kMultiply = 544,
  kScreen = 545,
  kDarken = 546,
  kLighten = 547,
  kColorDodge = 548,
  kColorBurn = 549,
  kHardLight = 550,
  kSoftLight = 551,
  kDifference = 552,
  kExclusion = 553,
  kHue = 554,
  kSaturation = 555,
  kColor = 556,
  kLuminosity = 557,
  kScaleDown = 558,
  kBalance = 559,
  kDrag = 560,
  kNoDrag = 561,
  kSpan = 562,
  kMinmax = 563,
  kEachLine = 564,
  kProgressive = 565,
  kInterlace = 566,
  kMarkers = 567,
  kInternalExtendToZoom = 568,
  kPanX = 569,
  kPanY = 570,
  kPanLeft = 571,
  kPanRight = 572,
  kPanUp = 573,
  kPanDown = 574,
  kManipulation = 575,
  kPinchZoom = 576,
  kLastBaseline = 577,
  kFirstBaseline = 578,
  kFirst = 579,
  kLast = 580,
  kSelfStart = 581,
  kSelfEnd = 582,
  kLegacy = 583,
  kSmooth = 584,
  kScrollPosition = 585,
  kRevert = 586,
  kUnset = 587,
  kLinearGradient = 588,
  kRadialGradient = 589,
  kConicGradient = 590,
  kRepeatingLinearGradient = 591,
  kRepeatingRadialGradient = 592,
  kRepeatingConicGradient = 593,
  kPaint = 594,
  kWebkitCrossFade = 595,
  kWebkitGradient = 596,
  kWebkitLinearGradient = 597,
  kWebkitRadialGradient = 598,
  kWebkitRepeatingLinearGradient = 599,
  kWebkitRepeatingRadialGradient = 600,
  kWebkitImageSet = 601,
  kFrom = 602,
  kTo = 603,
  kColorStop = 604,
  kRadial = 605,
  kAttr = 606,
  kCounter = 607,
  kCounters = 608,
  kRect = 609,
  kPolygon = 610,
  kFormat = 611,
  kInvert = 612,
  kGrayscale = 613,
  kSepia = 614,
  kSaturate = 615,
  kHueRotate = 616,
  kOpacity = 617,
  kBrightness = 618,
  kContrast = 619,
  kBlur = 620,
  kDropShadow = 621,
  kUrl = 622,
  kRgb = 623,
  kRgba = 624,
  kHsl = 625,
  kHsla = 626,
  kInternalLightDarkColor = 627,
  kMatrix = 628,
  kMatrix3d = 629,
  kPerspective = 630,
  kRotate = 631,
  kRotateX = 632,
  kRotateY = 633,
  kRotateZ = 634,
  kRotate3d = 635,
  kScale = 636,
  kScaleX = 637,
  kScaleY = 638,
  kScaleZ = 639,
  kScale3d = 640,
  kSkew = 641,
  kSkewX = 642,
  kSkewY = 643,
  kTranslate = 644,
  kTranslateX = 645,
  kTranslateY = 646,
  kTranslateZ = 647,
  kTranslate3d = 648,
  kX = 649,
  kY = 650,
  kZ = 651,
  kPath = 652,
  kRay = 653,
  kSides = 654,
  kCalc = 655,
  kWebkitCalc = 656,
  kMin = 657,
  kMax = 658,
  kClamp = 659,
  kMandatory = 660,
  kProximity = 661,
  kStyle = 662,
  kSize = 663,
  kAutoFill = 664,
  kAutoFit = 665,
  kVar = 666,
  kInternalVariableValue = 667,
  kEnv = 668,
  kAvoidPage = 669,
  kPage = 670,
  kRecto = 671,
  kVerso = 672,
  kAvoidColumn = 673,
  kP3 = 674,
  kRec2020 = 675,
  kTrue = 676,
  kFalse = 677,
  kNoPreference = 678,
  kDark = 679,
  kLight = 680,
  kOnly = 681,
  kReduce = 682,
  kActive = 683,
  kAliceblue = 684,
  kAntiquewhite = 685,
  kAquamarine = 686,
  kAzure = 687,
  kBeige = 688,
  kBisque = 689,
  kBlanchedalmond = 690,
  kBlueviolet = 691,
  kBrown = 692,
  kBurlywood = 693,
  kCadetblue = 694,
  kChartreuse = 695,
  kChocolate = 696,
  kCoral = 697,
  kCornflowerblue = 698,
  kCornsilk = 699,
  kCrimson = 700,
  kCyan = 701,
  kDarkblue = 702,
  kDarkcyan = 703,
  kDarkgoldenrod = 704,
  kDarkgray = 705,
  kDarkgreen = 706,
  kDarkgrey = 707,
  kDarkkhaki = 708,
  kDarkmagenta = 709,
  kDarkolivegreen = 710,
  kDarkorange = 711,
  kDarkorchid = 712,
  kDarkred = 713,
  kDarksalmon = 714,
  kDarkseagreen = 715,
  kDarkslateblue = 716,
  kDarkslategray = 717,
  kDarkslategrey = 718,
  kDarkturquoise = 719,
  kDarkviolet = 720,
  kDeeppink = 721,
  kDeepskyblue = 722,
  kDimgray = 723,
  kDimgrey = 724,
  kDodgerblue = 725,
  kFirebrick = 726,
  kFloralwhite = 727,
  kForestgreen = 728,
  kGainsboro = 729,
  kGhostwhite = 730,
  kGold = 731,
  kGoldenrod = 732,
  kGreenyellow = 733,
  kHoneydew = 734,
  kHotpink = 735,
  kIndianred = 736,
  kIndigo = 737,
  kIvory = 738,
  kKhaki = 739,
  kLavender = 740,
  kLavenderblush = 741,
  kLawngreen = 742,
  kLemonchiffon = 743,
  kLightblue = 744,
  kLightcoral = 745,
  kLightcyan = 746,
  kLightgoldenrodyellow = 747,
  kLightgray = 748,
  kLightgreen = 749,
  kLightgrey = 750,
  kLightpink = 751,
  kLightsalmon = 752,
  kLightseagreen = 753,
  kLightskyblue = 754,
  kLightslategray = 755,
  kLightslategrey = 756,
  kLightsteelblue = 757,
  kLightyellow = 758,
  kLimegreen = 759,
  kLinen = 760,
  kMagenta = 761,
  kMediumaquamarine = 762,
  kMediumblue = 763,
  kMediumorchid = 764,
  kMediumpurple = 765,
  kMediumseagreen = 766,
  kMediumslateblue = 767,
  kMediumspringgreen = 768,
  kMediumturquoise = 769,
  kMediumvioletred = 770,
  kMidnightblue = 771,
  kMintcream = 772,
  kMistyrose = 773,
  kMoccasin = 774,
  kNavajowhite = 775,
  kOldlace = 776,
  kOlivedrab = 777,
  kOrangered = 778,
  kOrchid = 779,
  kPalegoldenrod = 780,
  kPalegreen = 781,
  kPaleturquoise = 782,
  kPalevioletred = 783,
  kPapayawhip = 784,
  kPeachpuff = 785,
  kPeru = 786,
  kPink = 787,
  kPlum = 788,
  kPowderblue = 789,
  kRebeccapurple = 790,
  kRosybrown = 791,
  kRoyalblue = 792,
  kSaddlebrown = 793,
  kSalmon = 794,
  kSandybrown = 795,
  kSeagreen = 796,
  kSeashell = 797,
  kSienna = 798,
  kSkyblue = 799,
  kSlateblue = 800,
  kSlategray = 801,
  kSlategrey = 802,
  kSnow = 803,
  kSpringgreen = 804,
  kSteelblue = 805,
  kTan = 806,
  kThistle = 807,
  kTomato = 808,
  kTurquoise = 809,
  kViolet = 810,
  kWheat = 811,
  kWhitesmoke = 812,
  kYellowgreen = 813,
  kAlpha = 814,
  kLuminance = 815,
  kAccumulate = 816,
  kNew = 817,
  kSRGB = 818,
  kLinearrgb = 819,
  kButt = 820,
  kMiter = 821,
  kBevel = 822,
  kBeforeEdge = 823,
  kAfterEdge = 824,
  kCentral = 825,
  kTextBeforeEdge = 826,
  kTextAfterEdge = 827,
  kIdeographic = 828,
  kHanging = 829,
  kMathematical = 830,
  kUseScript = 831,
  kNoChange = 832,
  kResetSize = 833,
  kDynamic = 834,
  kNonScalingStroke = 835,
};

const int numCSSValueKeywords = 836;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
